/*
 * JBoss, the OpenSource J2EE webOS
 * 
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package services.ejb;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.util.List;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.jboss.logmanager.Level;
import org.jboss.logmanager.Logger;

import services.entities.DistributionCenter;
import services.entities.Region;
import services.entities.RegionPK;
import services.exceptions.CreateDataException;

@Stateless
public class DistributionCenterManagerBean implements DistributionCenterManager {

	private static Logger logger = Logger.getLogger(DistributionCenterManager.class.toString());
	
	@PersistenceContext(unitName="services")
	protected EntityManager entityManager;
	
	@PersistenceContext(unitName="batch-services")
	protected EntityManager batchEntityManager;

	@SuppressWarnings("unchecked")
	public List<Region> findAllRegions() {

		Query query = batchEntityManager.createNamedQuery("selectAllTheRegions");

		return query.getResultList();

	}

	@SuppressWarnings("unchecked")
	public List<DistributionCenter> findAllDistributionCenters() {

		Query query = batchEntityManager.createNamedQuery("selectAllTheDistributionCenters");

		return query.getResultList();

	}

	public void createDistributionCenters(String pathToRegionFile, int batchSize) throws CreateDataException {

		int rowsToFlush = 0, totalRows = 0;

		//  Create the distribution centers

		int[] distributionCenterIds = {10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500
				, 14000, 14500, 15000, 15500, 16000, 16500, 17000, 17500, 18000, 18500, 19000
				, 19500, 20000, 20500, 21000, 21500, 22000
		};

		String[] streets = {"10234 E. Side Street", "10432 W. Main Street", "11123 N. Left Street"
				, "11321 S. Left Street", "12234 W. Desert Street", "12432 W. Mountain Street"
				, "13456 W. Potato Street", "13564 N. Cold Street", "14123 N. Chicagoland Street"
				, "14321 E. Loui Street", "15789 E. Wheat Street", "15987 S. Oil Street"
				, "16123 S. Metroplex Street", "16321 S. Humid Street", "17456 S. Erosion Street"
				, "17654 S. Elvis Street", "18789 E. Colts Street", "18987 E. Reds Street"
				, "19123 E. Cheese Steak Street", "19321 S. Hot Street", "20345 S. Orange Street"
				, "20543 S. Chad Street", "21789 S. Bell Street", "21987 N. Atlantic Street"
				, "22123 N. Patriots Street"
		};

		String[] descriptions = {"Washington Distribution Center", "Oregon Distribution Center"
				, "Northern California Distribution Center", "Southern California Distribution Center"
				, "Arizona Distribution Center", "Colorado Distribution Center"
				, "Idaho Distribution Center", "Minnesota Distribution Center"
				, "Illinois Distribution Center", "Missouri Distribution Center"
				, "Kansas Distribution Center", "Oklahoma Distribution Center"
				, "North Texas Distribution Center", "South Texas Distribution Center"
				, "Louisianna Distribution Center", "Tennesse Distribution Center"
				, "Indiana Distribution Center", "Ohio Distribution Center"
				, "Pennsylvania Distribution Center", "Georgia Distribution Center"
				, "Northern Florida Distribution Center", "Southern Florida Distribution Center"
				, "Carolina Distribution Center", "New York Distribution Center"
				, "New England Distribution Center"
		};

		String[] cities = {"Seattle", "Porland", "San Francisco", "Los Angeles", "Phoenix"
				, "Denver", "Boise", "Minneapolis", "Chicago", "St. Louis", "Wichita", "Tulsa"
				, "Dallas", "Houston", "Baton Rouge", "Memphis", "Indianapolis", "Cincinatti"
				, "Philadephia", "Atlanta", "Tallahassee", "Miami", "Raleigh", "New York", "Boston"
		};

		String[] states = {"WA", "OR", "CA", "CA", "AZ", "CO", "ID", "MN", "IL", "MO", "KS", "OK"
				, "TX", "TX", "LA", "TN", "IN", "OH", "PA", "GA", "FL", "FL", "NC", "NY", "MA"
		};

		int[] zips = {98101, 97201, 94102, 90001, 85003, 80012, 83702, 55401, 60601, 63101, 67202
				, 74103, 75201, 77002, 70801, 38103, 46201, 45202, 19102, 30303, 32301, 33109
				, 27601, 10001, 2108
		};
		
		DistributionCenter distributionCenter = null;

		for(int i = 0; i < distributionCenterIds.length; i++) {
			distributionCenter = new DistributionCenter();

			distributionCenter.setAddressLine1(streets[i]);
			distributionCenter.setCity(cities[i]);
			distributionCenter.setDescription(descriptions[i]);
			distributionCenter.setDistributionCenterId(distributionCenterIds[i]);
			distributionCenter.setState(states[i]);
			distributionCenter.setZipCode(zips[i]);
			distributionCenter.setZipCodePlusFour(0);

			batchEntityManager.persist(distributionCenter);

			rowsToFlush++;

			if(rowsToFlush == batchSize) {
				batchEntityManager.flush();
				rowsToFlush = 0;
				//  Detach all entities to free memory
				batchEntityManager.clear();
				if(logger.isLoggable(Level.TRACE)) {
					logger.log(Level.TRACE, "Just flushed " + batchSize + " rows to the database.");
					logger.log(Level.TRACE, "Total rows flushed is " + (i+1));
				}
			}
		}

		//  Create the Regions for the Distribution Centers

		LineNumberReader regionDataFile = null;

		try {
			regionDataFile = new LineNumberReader(new FileReader(pathToRegionFile));
		} catch (FileNotFoundException e) {
			throw new CreateDataException("Couldn't open the region data file at: " + pathToRegionFile + ".", e);
		}

		String line = null;

		try {
			line = regionDataFile.readLine();
		} catch (IOException e) {
			throw new CreateDataException("Failed to read a line from the region data file: " + pathToRegionFile + ".", e);
		}

		Region region     = null;
		RegionPK regionPk = null;
		
		do {
			region   = new Region();
			regionPk = new RegionPK();

			String[] attributes = line.split("\t");

			regionPk.setDistributionCenterId(new Integer(attributes[0]));
			regionPk.setZipCode(new Integer(attributes[1]));

			region.setPrimaryKey(regionPk);

			region.setZipCodePlusFour(new Integer(attributes[2]));
			region.setCity(attributes[3]);
			region.setState(attributes[4]);

			batchEntityManager.persist(region);

			rowsToFlush++;
			totalRows++;

			if(rowsToFlush == batchSize) {
				batchEntityManager.flush();
				rowsToFlush = 0;
				//  Detach all the entities to free memory
				batchEntityManager.clear();
				if(logger.isLoggable(Level.TRACE)) {
					logger.log(Level.TRACE, "Just flushed " + batchSize + " rows to the database.");
					logger.log(Level.TRACE, "Total rows flushed is " + totalRows);
				}
			}
			
			region   = null;
			regionPk = null;
			
			try {
				line = regionDataFile.readLine();
			} catch (IOException e) {
				throw new CreateDataException("Failed to read a line from the region data file: " + pathToRegionFile + ".", e);
			}
		} while(line != null);

		return;
		
	}

}