package services.ejb;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.ejb.Timeout;
import javax.ejb.Timer;
import javax.ejb.TimerService;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.jboss.logmanager.Level;
import org.jboss.logmanager.Logger;

import services.ejb.OrderManager;
import services.entities.Address;
import services.entities.Customer;
import services.entities.Order;
import services.entities.OrderLine;
import services.exceptions.CreateDataException;
import services.exceptions.OrderEntryProcessFailedException;
import services.exceptions.PricingMethodNotFoundException;
import services.exceptions.ProductNotFoundException;

@Stateless
public class OrderManagerBean implements OrderManager {

	private static Logger logger = Logger.getLogger(OrderManagerBean.class.toString());
	
	@EJB
	private CustomerManager customerManager;
	
	@EJB
	private ProductManager productManager;
	
	@Resource
	TimerService timerService;
	
	@PersistenceContext (unitName="services")
	protected EntityManager entityManager;

	public Order createOrder(long customerId
			, List<OrderLine> orderLines) throws OrderEntryProcessFailedException {

		Customer customer = null;

		customer = customerManager.findByPrimaryKey(customerId);

		if (customer == null) {
			OrderEntryProcessFailedException orderProcessFailed = new OrderEntryProcessFailedException("Customer Not Found!");
			logger.log(Level.ERROR, "Customer customerId: " + customerId, orderProcessFailed);
			throw orderProcessFailed;
		}
		
		String addressLine2 = customer.getAddressLine2();

		if (addressLine2 == null) {
			addressLine2 = "";
		}

		Address shippingAddress = new Address();

		shippingAddress.setAddressLine1(customer.getAddressLine1());
		shippingAddress.setAddressLine2(addressLine2);
		shippingAddress.setCity(customer.getCity());
		shippingAddress.setState(customer.getState());
		shippingAddress.setZipCode(customer.getZipCode());
		shippingAddress.setZipCodePlusFour(customer.getZipCodePlusFour());

		Order newOrder = new Order();

		newOrder.setCustomerId(customer.getCustomerId());
		newOrder.setDistributionCenterId(customer.getDistributionCenterId());
		newOrder.setShippingAddressLine1(shippingAddress.getAddressLine1());
		newOrder.setShippingAddressLine2(shippingAddress.getAddressLine2());
		newOrder.setShippingCity(shippingAddress.getCity());
		newOrder.setShippingState(shippingAddress.getState());
		newOrder.setShippingZipCode(shippingAddress.getZipCode());
		newOrder.setShippingZipCodePlusFour(shippingAddress.getZipCodePlusFour());
		newOrder.setOrderDate(new Date());
		newOrder.setCustomer(customer);
		
		//  Price the order
		
		BigDecimal totalOrderAmount = new BigDecimal(0);
		
		long lineNumber = 1;

		for (OrderLine orderLine: orderLines) {
			
			orderLine.getOrderLinePK().setLineNumber(lineNumber);
			
			try {
				orderLine.setPrice(productManager.price(customer.getCustomerId(), orderLine.getProductId()));
			} catch(PricingMethodNotFoundException pricingMethodNotFound) {
				OrderEntryProcessFailedException orderProcessFailed = new OrderEntryProcessFailedException(pricingMethodNotFound);
				logger.log(Level.ERROR, "Pricing Method for customerId: " + customer.getCustomerId(), orderProcessFailed);
				throw orderProcessFailed;
			} catch(ProductNotFoundException productNotFound) {
				OrderEntryProcessFailedException orderProcessFailed = new OrderEntryProcessFailedException(productNotFound);
				logger.log(Level.ERROR, "Product not found for productId: " + orderLine.getProductId(), orderProcessFailed);
				throw orderProcessFailed;
			}

			orderLine.setExtendedPrice(productManager.extendedPrice(orderLine.getPrice(), orderLine.getQuantity()));

			totalOrderAmount = totalOrderAmount.add(orderLine.getExtendedPrice());
			lineNumber++;
		}
		
		newOrder.setTotalOrderAmount(totalOrderAmount);
		
		entityManager.persist(newOrder);
		
		String orderId = newOrder.getOrderId();
		
		for (OrderLine orderLine: orderLines) {
			orderLine.getOrderLinePK().setOrderId(orderId);
		}
		
		newOrder.setOrderLines(orderLines);

		entityManager.persist(newOrder);

		return newOrder;

	}
	
	public void createOrderPurgeTimer(boolean runWithTimer) throws CreateDataException {
		
		for (Object object : timerService.getTimers()) {
			Timer timer = (Timer) object;
			timer.cancel();
		}
		
		if (runWithTimer) {
			long twentyFourHours = 1000 * 60 * 60 * 24;
			timerService.createTimer(twentyFourHours, twentyFourHours, null);
		}
		
		return;
		
	}
	
	@Timeout
	public void purgeOldOrders(Timer timer) {
		
		logger.log(Level.INFO, "The purge Old Orders timer has expired: " + new Date());
		
		// Delete the previous days worth of orders
		
		Query query = entityManager.createNativeQuery("delete from `Order` where orderDate < :date");
		
		Calendar now = Calendar.getInstance();
		
		// Back up to the top of the hour, so we only delete orders from the previous period'
		
		now.set(Calendar.MINUTE, 0);
		now.set(Calendar.SECOND, 0);
		
		query.setParameter("date", new Date(now.getTimeInMillis()));
		query.executeUpdate();
		
		return;
		
	}

}
