package services.ejb;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.List;
import java.util.Random;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.jboss.logmanager.Level;
import org.jboss.logmanager.Logger;

import services.ejb.ProductManager;
import services.entities.PricingMethod;
import services.entities.Product;
import services.exceptions.CreateDataException;
import services.exceptions.PricingMethodNotFoundException;
import services.exceptions.ProductNotFoundException;

@Stateless
public class ProductManagerBean implements ProductManager {

	private static Logger logger = Logger.getLogger(ProductManagerBean.class.toString());

	@PersistenceContext (unitName="services")
	protected EntityManager entityManager;
	
	@PersistenceContext(unitName="batch-services")
	protected EntityManager batchEntityManager;

	public int numberOfProducts() {

		Long numberOfProducts = (Long) entityManager.createQuery("select count(*) from Product").getSingleResult();
		
		return numberOfProducts.intValue(); 

	}

	@SuppressWarnings("unchecked")
	public List<Product> findAllProducts() {

		Query query = batchEntityManager.createNamedQuery("selectAllTheProducts");

		return query.getResultList(); 

	}

	public void createProduct(Product product) {

		entityManager.persist(product);

	}

	public BigDecimal price(long customerId
			, long productId) throws PricingMethodNotFoundException, ProductNotFoundException {

		PricingMethod pricingMethod = null;
		Product product             = null;
		BigDecimal price            = new BigDecimal(0);

		pricingMethod = (PricingMethod) entityManager.find(PricingMethod.class
				, Long.valueOf(customerId));

		if (pricingMethod == null) {
			String message = "No pricing method found for customer: " + customerId;
			logger.log(Level.ERROR, message);
			throw new PricingMethodNotFoundException(message);
		}

		product = (Product) entityManager.find(Product.class
				, Long.valueOf(productId));

		if (product == null) {
			String message = "Cannot find the product: " + Long.valueOf(productId);
			logger.log(Level.ERROR, message);
			throw new ProductNotFoundException(message);
		}

		if (pricingMethod.isCostPlus()) {
			BigDecimal cost = product.getCost();
			BigDecimal percentPlus = pricingMethod.getPercent();
			price = (cost.add(cost.multiply(percentPlus)));
			price = price.setScale(2, RoundingMode.HALF_UP);
		} else {
			if (pricingMethod.isListLess()) {
				BigDecimal listPrice = product.getListPrice();
				BigDecimal percentOff = pricingMethod.getPercent();
				price = (listPrice.subtract(listPrice.multiply(percentOff)));
				price = price.setScale(2, RoundingMode.HALF_UP);
			}
		}

		return price;

	}

	public BigDecimal extendedPrice(BigDecimal price
			, long quantity) {

		return price.multiply(new BigDecimal(quantity)).setScale(2);

	}

	public long whatIsTheDemand(long productId
			, long distributionCenterId) {
		
		Query query = entityManager.createNamedQuery("oldestOrderDateByProductAndDistributionCenter");

		query.setParameter("productId", productId);
		query.setParameter("distributionCenterId", distributionCenterId);
		
		Date oldestOrderDate = (Date) query.getSingleResult();
		
		if (oldestOrderDate == null) {
			// Hey there are no orders for this product/distribution center, so we can return 0
			return 0;
		}
		
		query = entityManager.createNamedQuery("newestOrderDateByProductAndDistributionCenter");
		
		query.setParameter("productId", productId);
		query.setParameter("distributionCenterId", distributionCenterId);
		
		Date newestOrderDate = (Date) query.getSingleResult();

		long totalDemand = 0, daysDemand = 0, daysOfDemand;
		
		query = entityManager.createNamedQuery("demandByProductAndDistributionCenter");
		
		query.setParameter("productId", productId);
		query.setParameter("distributionCenterId", distributionCenterId);
		
		totalDemand = (Long) query.getSingleResult();
		
		daysOfDemand = (newestOrderDate.getTime() - oldestOrderDate.getTime()) * 1000 * 60 * 60 * 24;
		
		if(daysOfDemand <= 1) {
			daysDemand = totalDemand;
		} else {
			daysDemand = totalDemand / daysOfDemand;
		}

		return daysDemand;

	}

	public void createProducts(int numberOfProducts, int batchSize)
		throws CreateDataException {

		if(numberOfProducts() > 0) {
			throw new CreateDataException("There are already products in the Product table!");
		}
		
		int rowsToFlush = 0;

		Random lvalue = new Random(System.currentTimeMillis());
		Random rvalue = new Random(System.currentTimeMillis());

		BigDecimal zero         = new BigDecimal("00.00");
		BigDecimal fiftyCents   = new BigDecimal("00.50");
		BigDecimal multiplicand = new BigDecimal("2");

		Product product = null;
		
		for(int i = 0; i < numberOfProducts; i++) {
			product = new Product();

			product.setProductId(i+1);
			product.setDescription("Test Product " + (i+1));

			String costString = ((Integer) lvalue.nextInt(99)).toString()
				+ "."
				+ ((Integer) rvalue.nextInt(99)).toString();

			BigDecimal cost = new BigDecimal(costString);
			cost = cost.setScale(2);

			if(cost.compareTo(zero) == 0) {
				cost = cost.add(fiftyCents);
			}

			BigDecimal listPrice = cost.multiply(multiplicand);

			product.setCost(cost);
			product.setListPrice(listPrice);

			batchEntityManager.persist(product);

			rowsToFlush++;

			if(rowsToFlush == batchSize) {
				batchEntityManager.flush();
				rowsToFlush = 0;
				batchEntityManager.clear();
				if(logger.isLoggable(Level.TRACE)) {
					logger.log(Level.TRACE, "Just flushed " + batchSize + " rows to the database.");
					logger.log(Level.TRACE, "Total rows flushed is " + (i+1));
				}
			}
		}
		
		return;

	}

}
