/*
 * JBoss, the OpenSource J2EE webOS
 * 
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package services.web;

import javax.naming.InitialContext;
import javax.servlet.http.HttpServlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.jboss.logmanager.Level;
import org.jboss.logmanager.Logger;

import services.ejb.OrderInquiry;
import services.entities.Order;
import services.entities.OrderLine;

/**
 * Servlet Class
 *
 * This OrderInquiryServlet provides the method for processing an HTTP
 * Get request asking for the order history of a specific customer.  It
 * does this by getting the customerId from the query string, and calling
 * the findOrdersByCustomer method of the OrderInquiry Stateful Session
 * Bean.  It also keeps track, using the HTTP session, of the pagination
 * of the results, and whether the request is for the next page, and the
 * initial query has been run already.
 *  
 */
public class OrderInquiryServlet extends HttpServlet {

	/** The serialVersionUID */
	private static final long serialVersionUID = 1L;
	private static Logger logger = Logger.getLogger(OrderInquiryServlet.class.toString());

	public OrderInquiryServlet() {

		super();

	}

	public void init(ServletConfig config) throws ServletException {

		super.init(config);

	}

	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		response.setContentType("text/html");
		PrintWriter out = response.getWriter();

		HttpSession session = request.getSession();

		OrderInquiry orderInquiry = (OrderInquiry) session.getAttribute("sfsbOrderInquiry");

		if (orderInquiry == null) {

			InitialContext context = null;

			try {
				context = new InitialContext();
				orderInquiry = (OrderInquiry) context.lookup("java:app/OrderManagerEJB/OrderInquiryBean!services.ejb.OrderInquiry");
				session.setAttribute("sfsbOrderInquiry", orderInquiry);
			} catch(Exception lookupError) {
				logger.log(Level.ERROR, lookupError.getStackTrace().toString());
				response.setStatus(500);
				throw new ServletException(lookupError);
			}

		}

		try {
			long customerId = Long.parseLong(request.getParameter("customerId"));

			List<Order> orders = orderInquiry.findOrdersByCustomer(customerId);

			out.println("<html><head><title>");
			out.println("Customers Order History");
			out.println("</title></head>");
			out.println("<body>");

			out.println("<h1>");
			out.println("Orders");
			out.println("</h1>");

			int pagination = orderInquiry.getPaginationForInquiries();

			int orderCount = 0;

			for (Order order: orders) {
				out.println("<p>");
				out.print("Order Id: ");
				out.print(order.getOrderId());
				out.print(", Total Order Amount: ");
				out.print(order.getTotalOrderAmount());
				out.println(", Date Purchased: ");
				out.println(order.getOrderDate());
				out.println("<p>");
				out.println("Shipped To: ");
				out.println(order.getShippingAddressLine1());
				out.println(" " + order.getShippingCity() + ",");
				out.println(" " + order.getShippingState() + " ");
				out.println(" " + order.getShippingZipCode() + " ");
				out.println("<p>");
				List<OrderLine> orderLines = order.getOrderLines();
				for (OrderLine orderLine: orderLines) {
					out.println("Line Number: ");
					out.println(orderLine.getOrderLinePK().getLineNumber());
					out.println(", Product: ");
					out.println(orderLine.getProductId());
					out.println(", Price: ");
					out.println(orderLine.getPrice());
					out.println(", Quantity: ");
					out.println(orderLine.getQuantity());
					out.println(", Extended Price: ");
					out.println(orderLine.getExtendedPrice());
					out.println("<br>");
				}
				out.println("</p>");
				orderCount++;
			}

			if (orderCount == pagination) {
				//  Hey, there might be more orders, so give a next link
				out.println("<p>");
				out.print("<a href=/Order/OrderInquiry?customerId=");
				out.print(request.getParameter("customerId"));
				out.println(">Next page...</a>");
			}

			out.println("<p>");
			out.print("<a href=/Order>Enter more orders...</a>");
			out.println("</p>");
		} catch (Exception e) {
			logger.log(Level.ERROR, "Error in doGet of OrderInquiryServlet: ", e);
			logger.log(Level.ERROR, e.getStackTrace().toString());
			response.setStatus(500);
			throw new ServletException(e);
		} finally {
			out.println("</body></html>");
			out.close();
		}

	}

}
